/*
 * config.c
 * Copyright (C) Steffen Pohle 2009 <steffen@gulpe.de>
 * 
 * main.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * main.c is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#if defined(__MINGW32CE__) || defined(_WIN32_WCE) || defined(__MINGW32__)
  #include <windows.h>
  #include <commctrl.h>
  #include <winbase.h>
  #include <shlobj.h>
  #ifndef GOSMROUTE
    #include <aygshell.h>
  #endif
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include "osmroute.h"

#if defined(GOSMROUTE) || defined(WOSMROUTE)
#include "draw.h"
#endif


/*
 * some variabled which have to be present in all apps.
 */
int app_status = APPSTATUS_nothing;
// float debug_lon = 11.3500, debug_lat = 48.7500;
char configpath[LEN_FILENAME];
char datapath[LEN_FILENAME] = "map/";
char cachepath[LEN_FILENAME] = "cache/";
char logpath[LEN_FILENAME] = "logs/";


void config_init () {
	struct stat sb;

#if defined(__MINGW32CE__) || defined(_WIN32_WCE)
	strcpy (configpath, "\\My Documents\\wOSMroute\\");
	strcpy (logpath, "\\My Documents\\wOSMroute\\Logdir\\");
#endif
#if defined(__MINGW32__) && !defined(__MINGW32CE__) && !defined(_WIN32_WCE)
	TCHAR szPath[MAX_PATH];
	if(SUCCEEDED(SHGetFolderPath(NULL, CSIDL_PERSONAL|CSIDL_FLAG_CREATE, NULL, 0, szPath))) {
		snprintf (configpath, LEN_FILENAME, "%s\\wOSMroute\\", szPath);
		snprintf (logpath, LEN_FILENAME, "%s\\wOSMroute\\Logdir\\", szPath);
	}
#endif
#if !defined(__MINGW32CE__) && !defined(_WIN32_WCE) && !defined(__MINGW32__)
	char *hd;

    if ((hd = getenv ("HOME")) == NULL) {
        /* Homedir konnte nicht ermittelt werden. */
		strcpy (configpath, ".OSMroute/");
		strcpy (logpath, ".OSMroute/Logdir/");
    }
    else {
		snprintf (configpath, LEN_FILENAME, "%s/.OSMroute/", hd);
		snprintf (logpath, LEN_FILENAME, "%s/.OSMroute/Logdir/", hd);
	}
#endif

	d_print_init();
	if (stat(configpath, &sb) == -1) {
		char fn[LEN_FILENAME];
		strncpy (fn, configpath, LEN_FILENAME);
		fn[strlen(fn)-1] = 0;
		d_printf ("create config dir:%s", fn);
#if !defined(__MINGW32CE__) && !defined(_WIN32_WCE) && !defined(__MINGW32__)
		mkdir (fn, 0755);
#else
		mkdir (fn);
#endif
	}

	if (stat(logpath, &sb) == -1) {
		char fn[LEN_FILENAME];
		strncpy (fn, logpath, LEN_FILENAME);
		fn[strlen(fn)-1] = 0;
		d_printf ("create config dir:%s", fn);
#if !defined(__MINGW32CE__) && !defined(_WIN32_WCE) && !defined(__MINGW32__)
		mkdir (fn, 0755);
#else
		mkdir (fn);
#endif
	}

#if defined(GOSMROUTE) || defined(WOSMROUTE)
	config_draw_init ();
#endif							// GOSMROUTE
	app_status = APPSTATUS_nothing;
};


/*
 * load config files
 */
void config_load () {
	char filename[LEN_FILENAME];
    char buf[1024];
    char *findit, *keyword, *value;
    int i;
	FILE *f;
	
	sprintf (filename, "%sconfig.cfg", configpath);
	d_printf ("load config file:%s", filename);
	f =  fopen (filename, "r");
	if (f) {
		while (fgets (buf, sizeof (buf), f) != NULL) {
			findit = strchr (buf, '\n');
			if (findit) findit[0] = '\0';
			if (buf[0] == '\0') continue;

			keyword = buf;
			while (isspace (*keyword)) keyword++;

			value = strchr (buf, '=');
			if (value == NULL) continue;

			*value = 0;
			value++;
			while (*value == ' ') value++;
			while (keyword[strlen (keyword) - 1] == ' ') keyword[strlen (keyword) - 1] = 0;
			while (value[strlen (value) - 1] == ' ') value[strlen (value) - 1] = 0;
			if (strlen (value) == 0) continue;

			for (i = 0; i < (int) strlen (keyword); i++)
				keyword[i] = tolower (keyword[i]);
			
			if (!strcmp (keyword, "datapath")) 
				strncpy (datapath, value, LEN_FILENAME);
#if defined(GOSMROUTE) || defined(WOSMROUTE)
	        if (!strcmp (keyword, "view_lon"))
				view_lon = atof (value);
	        if (!strcmp (keyword, "view_lat"))
				view_lat = atof (value);
	        if (!strcmp (keyword, "view_scale"))
				view_scale = atoi (value);
			if (!strcmp (keyword, "gps_device")) 
				gps_set_device (value);
#endif
		}
		fclose (f);
	}

//	sprintf (filename, "%s/default.style", datapath);
//	config_draw_load (filename);

	d_printf ("configdir: %s", configpath);
	d_printf ("datadir: %s", datapath);
	d_printf ("logdir: %s", logpath);
	d_printf ("cachedir: %s", cachepath);
};


/*
 * save config file
 */
void config_save () {
	char filename[LEN_FILENAME];
	FILE *f;

	sprintf (filename, "%sconfig.cfg", configpath);
	d_printf ("save config file:%s", filename);
	f = fopen (filename, "w");
	if (f) {
		fprintf (f, "# config file for osmroute\n\n");
		fprintf (f, "datapath=%s\n", datapath);
#if defined(GOSMROUTE) || defined(WOSMROUTE)
		fprintf (f, "view_lon=%f\n", view_lon);
		fprintf (f, "view_lat=%f\n", view_lat);
		fprintf (f, "view_scale=%d\n", view_scale);
		fprintf (f, "gps_device=%s\n", gps_get_device ());
#endif
		fclose (f);
	}

//	sprintf (filename, "%s/default.style", datapath);
//	config_draw_save (filename);
};

#if defined(GOSMROUTE) || defined(WOSMROUTE)
void config_draw_init () {
	int n;

	for (n = 0; n < MAREA_MAX; n++) {
		draw_type[n] = 1;
	}

	color_white = color_alloc (0xFFFF, 0xFFFF, 0xFFFF);
	color_black = color_alloc (0x0000, 0x0000, 0x0000);
	color_grey = color_alloc (0x8000, 0x8000, 0x8000);
	color_red1 = color_alloc (0x8000, 0x0000, 0x0000);
	color_red2 = color_alloc (0xFFFF, 0x0000, 0x0000);
	color_green1 = color_alloc (0x0000, 0x8000, 0x0000);
	color_green2 = color_alloc (0x0000, 0xFFFF, 0x0000);
	color_blue1 = color_alloc (0x0000, 0x0000, 0x8000);
	color_blue2 = color_alloc (0x0000, 0x0000, 0xFFFF);
	color_pink = color_alloc (0xFFFF, 0x8000, 0xFFFF);
	color_bg = color_alloc (0,0,0);					 // background
	color_fg = color_alloc (0x8000,0x8000,0x8000);   // foreground

	for (n = 0; n < MWAY_MAX; n++) {
		draw_linestyle[n].width = 0.0;
		draw_linestyle[n].borderc = color_alloc (0x8000, 0x8000,0x8000);

		switch (n) {
		case (MWAY_highway):
			draw_linestyle[n].width = 4.0;
			draw_linestyle[n].c = color_alloc (0xFFFF,0xFFFF,0x8000);
			break;
		case (MWAY_residential):
			draw_linestyle[n].width = 2.0;
			draw_linestyle[n].c = color_alloc (0xA000,0xA000,0xA000);
			break;
		case (MWAY_tertiary):
			draw_linestyle[n].width = 3.0;
			draw_linestyle[n].c = color_alloc (0xA000,0xA000,0x0000);
			break;
		case (MWAY_cycleway):
			draw_linestyle[n].c = color_alloc (0x4000,0xA000,0x4000);
			break;
		case (MWAY_footway):
			draw_linestyle[n].c = color_alloc (0x4000,0x4000,0xA000);
			break;
		case (MWAY_track):
			draw_linestyle[n].c = color_alloc (0x8000,0x8000,0x8000);
			break;
		case (MWAY_service):
			draw_linestyle[n].c = color_alloc (0x6000,0x6000,0x6000);
			break;
		case (MWAY_unknown):
		default:
			draw_linestyle[n].c = color_alloc (0x4000,0x4000,0x4000);
			break;
		}
	}

	for (n = 0; n < MAREA_MAX; n++) switch (n) {
		case (MAREA_water):
			color_polygon[n] = color_alloc (0x4000,0x4000,0xFFFF);
			break;
		case (MAREA_greenfield):
			color_polygon[n] = color_alloc (0x2000,0x8000,0x2000);
			break;
		case (MAREA_wood):
			color_polygon[n] = color_alloc (0x0000,0x6000,0x0000);
			break;
		case (MAREA_land):
			color_polygon[n] = color_alloc (0x6000,0x6000,0x0000);
			break;
		case (MAREA_residential):
			color_polygon[n] = color_alloc (0x4000,0x4000,0x4000);
			break;
		case (MAREA_farm):
			color_polygon[n] = color_alloc (0x8000,0x4000,0x2000);
			break;
		case (MAREA_industrial):
			color_polygon[n] = color_alloc (0x8000,0x6000,0x8000);
			break;
		case (MAREA_commercial):
			color_polygon[n] = color_alloc (0x8000,0x0000,0x8000);
			break;
		case (MAREA_military):
			color_polygon[n] = color_alloc (0x2000,0x2000,0x2000);
			break;
		default:
			color_polygon[n] = color_alloc (0xFFFF,0x0000,0x0000);
			break;
	}	

	for (n = 0; n < DRAW_BTN_MAX; n++) draw_btn[n] = &color_grey;
};
#endif					// GOSMROUTE

