/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * convert_main.c
 * Copyright (C) Steffen Pohle 2009 <steffen@gulpe.de>
 * 
 * main.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * main.c is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include <math.h>
#include <limits.h>

#include "memoryleak.h"
#include "map.h"
#include "convert_port.h"


char curfile[LEN_FILENAME];

/* call this function always at the end of programm */
void main_wnd_quit() {
}


int main (int argc, char *argv[]) {
	struct stat sb;
	int i = 0, c;
	struct map_pos p;
	float lon, lat;
	
	d_print_init();
	osm_delete_tmpdir ("map");
	
	ml_init();
#ifdef ENABLE_NLS
	bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
	textdomain (GETTEXT_PACKAGE);
#endif

	/* check for the cache path and the map path */
	if (stat(cachepath, &sb) == -1) {
		mkdir (cachepath, 0755);
	}

	if (stat(datapath, &sb) == -1) {
		mkdir (datapath, 0755);
	}
	
	config_init ();
	config_load ();

	map_init ();
	main_wnd_startall ();
	
	if (argc > 1) {
		strncpy (curfile, argv[1], LEN_FILENAME);
		osm_loadfile (curfile);
	}
	map_save_all();
	d_printf ("");
	return 0;
};


void main_wnd_startall () {
};


void main_wnd_loop (long long int cur, long long int max) {
	struct memoryleak_info mi;
	static long long int last_line = 0;
	static time_t timeout = 0;
	
	if (timeout+1 > time (NULL)) return;
	timeout = time(NULL);

	ml_get_stats (&mi);
	printf("pos:%5.3f %%  nodes:%lld   ways:%lld   ", osm_file_pos, osm_nodes_cnt, osm_ways_cnt);

	if (mi.size < 10*1024)	printf("%lld bytes ", mi.size);
	else if (mi.size < 10*1024*1024) printf("%lld kbytes ", mi.size/1024);
	else printf("%lld Mbytes ", mi.size/(1024*1024));
	printf("blocks:%lld \n", mi.blocks);
};


void main_wnd_status (char *text, int progress) {
};


void main_wnd_update () {
};

void draw () {
};
