/***************************************************************************
 *            gtk_gfx.c
 *
 *  Copyright  2010  Steffen Pohle
 *  steffen@gulpe.de
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "map.h"
#include "gtk_port.h"
#include "memoryleak.h"

extern GtkWidget *map_da;
extern GdkGC  *gc;
extern GdkGCValues gcvalues;
extern GdkColormap *colormap;
extern iPoint draw_screensize;

/************************************************************************
 * functions for starting and finishing the drawing process
 */
static int dont_draw = 0;	// prevent from drawing during drawing calls
int draw_start () {
	int i;

	/* check that we aren't drawing at the moment */
	if (dont_draw) return FALSE;
	dont_draw = TRUE;

	if (map_da == NULL) return FALSE;
	gc = gdk_gc_new (map_da->window);
	if (gc == NULL) return FALSE;
	gdk_gc_get_values (gc, &gcvalues);
	colormap = gdk_colormap_get_system ();

	color_white = color_alloc (0xFFFF, 0xFFFF, 0xFFFF);
	color_black = color_alloc (0x0000, 0x0000, 0x0000);
	color_grey = color_alloc (0x8000, 0x8000, 0x8000);
	color_red1 = color_alloc (0x8000, 0x0000, 0x0000);
	color_red2 = color_alloc (0xFFFF, 0x0000, 0x0000);
	color_green1 = color_alloc (0x0000, 0x8000, 0x0000);
	color_green2 = color_alloc (0x0000, 0xFFFF, 0x0000);
	color_blue1 = color_alloc (0x0000, 0x0000, 0x8000);
	color_blue2 = color_alloc (0x0000, 0x0000, 0xFFFF);
	color_pink = color_alloc (0xFFFF, 0x8000, 0xFFFF);
	color_fg = color_alloc (color_fg.r, color_fg.g, color_fg.b);
	color_bg = color_alloc (color_bg.r, color_bg.g, color_bg.b);

	for (i = 0; i < MWAY_MAX; i++) {
		draw_linestyle[i].c = color_alloc (draw_linestyle[i].c.r, draw_linestyle[i].c.g, draw_linestyle[i].c.b);
		draw_linestyle[i].borderc = color_alloc (draw_linestyle[i].borderc.r, draw_linestyle[i].borderc.g, draw_linestyle[i].borderc.b);
	}
	for (i = 0; i < MAREA_MAX; i++) 
		color_polygon[i] = color_alloc (color_polygon[i].r, color_polygon[i].g, color_polygon[i].b);
//	_linescnt = 0;
//	_lineswarning = 0;
	
	return TRUE;
};


void draw_stop () {
	gdk_gc_unref (gc);
	colormap = NULL;
	dont_draw = 0;
};


struct color color_alloc (unsigned short int r, unsigned short int g, unsigned short int b) {
	struct color c;

	c.r = r;
	c.b = b;
	c.g = g;
	
	if (colormap != NULL) {
		c.c.red = r;
		c.c.blue = b;
		c.c.green = g;
		gdk_colormap_alloc_color (colormap, &c.c, TRUE, TRUE);
	}

	return c;
};


void draw_getsize (int *x, int *y) {
	*x = map_da->allocation.width;
	*y = map_da->allocation.height;
};


/***************************************************************************
 * img functions
 */
struct image *gfx_img_alloc (int w, int h) {
	struct image *img = (struct image*) ml_malloc (sizeof (struct image));

	img->pixmap = gdk_pixmap_new (map_da->window, w, h, -1);
	img->gc = gdk_gc_new (img->pixmap);
	img->width = w;
	img->height = h;

	return img;
};


void gfx_img_free (struct image *img) {
	gdk_gc_unref (img->gc);
	g_object_unref (img->pixmap);
	ml_free (img);
};


void gfx_img_clear (struct image *img, struct color *c) {
  gdk_gc_set_foreground (img->gc,&c->c);

  gdk_draw_rectangle (img->pixmap, img->gc, TRUE, 0, 0, img->width, img->height);

};


void gfx_draw_img (int dx, int dy, int dw, int dh, struct image *simg, int sx, int sy) {
	if (simg == NULL || simg->pixmap == NULL) {
		d_printf ("WARN: gfx_draw_img drawing on NULL image");
		d_print_backtrace ();
		return;
	}

	/* check for boundrys, do clipping */
	if (sx + dw > simg->width) dw = simg->width - sx;
	if (dx + dw > draw_screensize.x) dw = draw_screensize.x - dx;
	if (sy + dh > simg->height) dh = simg->height - sy;
	if (dh + dh > draw_screensize.y) dh = draw_screensize.y - dy;
	if (dw <= 0 || dh <= 0) return;

	/* draw the pixmap onto the screen */
	gdk_draw_drawable (map_da->window, gc, simg->pixmap, sx, sy, dx, dy, dw, dh);
};


void gfx_img_draw_img (struct image *dimg, int dx, int dy, int dw, int dh, struct image *simg, int sx, int sy) {
	if (simg == NULL || simg->pixmap == NULL || dimg == NULL || dimg->pixmap == NULL) {
		d_printf ("WARN: gfx_img_draw_img drawing on NULL or from NULL image");
		d_print_backtrace ();
		return;
	}

	/* check for boundrys, do clipping */
	if (sx + dw > simg->width) dw = simg->width - sx;
	if (dx + dw > dimg->width) dw = dimg->width - dx;
	if (sy + dh > simg->height) dh = simg->height - sy;
	if (dh + dh > draw_screensize.y) dh = dimg->height - dy;
	if (dw <= 0 || dh <= 0) return;

	/* draw the pixmap onto the screen */
	gdk_draw_drawable (dimg->pixmap, dimg->gc, simg->pixmap, sx, sy, dx, dy, dw, dh);
};


void gfx_img_draw_line (struct image *img, int x1, int y1, int x2, int y2, struct line_style style) {
	gdk_gc_set_foreground (img->gc,&style.c.c);
	gdk_gc_set_line_attributes (img->gc, style.width, GDK_LINE_SOLID, GDK_CAP_ROUND, GDK_JOIN_ROUND);
	gdk_draw_line(img->pixmap, img->gc, x1, y1, x2, y2);
};


void gfx_img_draw_text (struct image *img, int x, int y, char *text, struct color *c) {
	PangoLayout *layout = gtk_widget_create_pango_layout(map_da, NULL);
	pango_layout_set_text(layout, text, -1);

	gdk_gc_set_foreground (img->gc, &color_grey.c);
	gdk_draw_layout(img->pixmap, img->gc, x-1, y-1, layout);
	gdk_draw_layout(img->pixmap, img->gc, x-1, y+1, layout);
	gdk_draw_layout(img->pixmap, img->gc, x+1, y-1, layout);
	gdk_draw_layout(img->pixmap, img->gc, x+1, y+1, layout);
	
	gdk_gc_set_foreground (img->gc, &c->c);
	gdk_draw_layout(img->pixmap, img->gc, x, y, layout);
};


void gfx_img_draw_polygon (struct image *img, iPoint *p, int pcnt, struct line_style style, struct color c) {
	static GdkPoint *polygon = NULL;
	static int polygon_cnt = 0;
	int i;

	if (polygon_cnt < pcnt) {
		polygon_cnt = pcnt;
		if (polygon != NULL)
			polygon = (GdkPoint *) ml_realloc (polygon, sizeof (GdkPoint) * polygon_cnt);
		else
			polygon = (GdkPoint *) ml_malloc (sizeof (GdkPoint) * polygon_cnt);
	}

	for (i = 0; i < pcnt; i++) {
		polygon[i].x = p[i].x;
		polygon[i].y = p[i].y;
	}

	gdk_gc_set_foreground (img->gc, &c.c);

	gdk_draw_polygon (img->pixmap, img->gc, TRUE, polygon, pcnt);

};

