/***************************************************************************
 *            gtk_wnd_main.c
 *
 *  Thu Jul 19 19:00:10 2007
 *  Copyright  2007  Steffen Pohle
 *  steffen@gulpe.de
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include "map.h"
#include "gtk_port.h"
#include "memoryleak.h"

GtkWidget *main_wnd;
GtkWidget *statusbar;
GtkWidget *scale;
GtkWidget *positionlon;
GtkWidget *positionlat;
GtkWidget *search;
GtkWidget *progressbar;
GtkToggleAction *gpsenabled;

extern GtkActionGroup *main_menu_ac;
gint status_bar_contextid;


void main_wnd_enable (int enabled) {
	gtk_action_set_sensitive (gtk_action_group_get_action (main_menu_ac, "AppMenu"), enabled);
	gtk_action_set_sensitive (gtk_action_group_get_action (main_menu_ac, "AppQuit"), enabled);
	gtk_action_set_sensitive (gtk_action_group_get_action (main_menu_ac, "AppPref"), enabled);
	gtk_action_set_sensitive (gtk_action_group_get_action (main_menu_ac, "MapMenu"), enabled);
	gtk_action_set_sensitive (gtk_action_group_get_action (main_menu_ac, "MapDownload"), enabled);
	gtk_action_set_sensitive (gtk_action_group_get_action (main_menu_ac, "MapRefresh"), enabled);
	gtk_action_set_sensitive (gtk_action_group_get_action (main_menu_ac, "MapInfo"), enabled);
	gtk_action_set_sensitive (gtk_action_group_get_action (main_menu_ac, "HelpMenu"), enabled);
	gtk_action_set_sensitive (gtk_action_group_get_action (main_menu_ac, "HelpAbout"), enabled);
};


/******************************************************************************
 * update stausbar 
 */
void main_wnd_status (char *text, int progress) {
	static char oldtext[255];
	char sbtext[255];
	struct memoryleak_info m;
	
	/* scale */
	gtk_combo_box_set_active (GTK_COMBO_BOX(scale), view_scale);

	/* progress bar */
	if (progress != -2) {
		if (progress != -1)
			gtk_progress_bar_pulse (GTK_PROGRESS_BAR(progressbar));
		else
			gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR(progressbar), 1.0);

		if (text) strncpy (oldtext, text, 255);
		snprintf (sbtext, 255, "%s\0", oldtext);
		gtk_progress_bar_set_text (GTK_PROGRESS_BAR(progressbar), sbtext);
	}

	/* gps enabled menubutton? */
	gtk_toggle_action_set_active (gpsenabled, gps_isrunning ());
	ml_get_stats (&m);
};


/*
 * scale widget changed.
 */
gboolean main_wnd_scale_change (GtkWidget *widget, gpointer data) {
	draw_setscale (gtk_combo_box_get_active (GTK_COMBO_BOX(scale)));
	draw ();
	return FALSE;
};


gboolean main_wnd_position_activate (GtkEntry *entry, gpointer data) {
	gchar *lon, *lat;
	float flon, flat;

	lon = (gchar *) gtk_entry_get_text (GTK_ENTRY(positionlon));
	lat = (gchar *) gtk_entry_get_text (GTK_ENTRY(positionlat));

	if (map_text2geo (lon, &flon) && map_text2geo (lat, &flat)) {
		view_lon = flon;
		view_lat = flat;
		draw ();
	}
	
	return FALSE;
};


/*
 * check if we can quit the programm 
 */
static gboolean main_wnd_delete_event(GtkWidget *widget, GdkEvent *event, gpointer data) {
    return FALSE; /* we can quit */
};


/*
 * callback function for the cyclic loop
 */
gboolean main_wnd_cyclicloop (gpointer data) {
	struct gps_data *gpspos;
	struct memoryleak_info mi;

	while (gtk_events_pending())		// make sure we handle also other GTK events..
		gtk_main_iteration();

	if (gps_isrunning () > 0) {
		if (gpspos = gps_loop ()) {
			draw_set_gpspos (gpspos);
		}
	}

	map_da_button_stillpressed (); /* needed to check for long button pressing */
	
	return TRUE;
};


/*
 * to make sure the gtk subsystem will quit, without it the
 * windows just disapears but the programm is still in memory 
 */
void main_wnd_destroy (GtkWidget *widget, gpointer   data) {
	main_wnd_enable (FALSE);
	main_wnd_status (_("save all..."), -1);
	map_save_all ();
	gtk_main_quit ();
};


/**********************************************************************
 *
 * the main window..
 *
 */

/*
 * creates the main window 
 */
GtkWidget *main_wnd_create () {
	GtkWidget *window;
	GtkWidget *main_vbox;
	GtkWidget *widget;
	GtkWidget *box;
	GtkWidget *label;
	int i;
	char txt[256];

	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title (GTK_WINDOW(window), _("gOSMroute"));
	gtk_window_set_default_size (GTK_WINDOW(window), 600, 400);

	g_signal_connect (G_OBJECT (window), "delete_event", G_CALLBACK (main_wnd_delete_event), NULL);
	g_signal_connect (G_OBJECT (window), "destroy", G_CALLBACK (main_wnd_destroy), NULL);

	main_vbox = gtk_vbox_new (FALSE, 1);
	gtk_container_set_border_width (GTK_CONTAINER (main_vbox), 1);
	gtk_container_add (GTK_CONTAINER (window), main_vbox);

	/*
	 * create our menu..
	 */
	gtk_box_pack_start (GTK_BOX (main_vbox), main_menu_create(window), FALSE, TRUE, 0);

	/*
	 * create the main part of the window (DRAWAREA)
	 */
	gtk_box_pack_start (GTK_BOX (main_vbox), map_da_create (), TRUE, TRUE, 0);

	/*
	 * create pos, search, scale, progressbar status bar
	 */
	box = gtk_hbox_new (FALSE, 1);

	/* pos */
	label = gtk_label_new (_("Position Lon:"));
	gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 4);
	positionlon = gtk_entry_new ();
	gtk_box_pack_start (GTK_BOX (box), positionlon, FALSE, FALSE, 4);
	gtk_widget_set_size_request (positionlon, 75, -1);
	g_signal_connect (G_OBJECT (positionlon), "activate", G_CALLBACK (main_wnd_position_activate), NULL);

	label = gtk_label_new (_("Lat:"));
	gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 4);
	positionlat = gtk_entry_new ();
	gtk_box_pack_start (GTK_BOX (box), positionlat, FALSE, FALSE, 4);
	gtk_widget_set_size_request (positionlat, 75, -1);
	g_signal_connect (G_OBJECT (positionlat), "activate", G_CALLBACK (main_wnd_position_activate), NULL);

	widget = gtk_vseparator_new();
	gtk_box_pack_start (GTK_BOX (box), widget, FALSE, FALSE, 4);
	
	/* search */
	label = gtk_label_new (_("Search:"));
	gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 4);
	search = gtk_combo_box_entry_new ();
	gtk_box_pack_start (GTK_BOX (box), search, FALSE, FALSE, 4);
	widget = gtk_vseparator_new();
	gtk_box_pack_start (GTK_BOX (box), widget, FALSE, FALSE, 4);
	
	/* scale */
	label = gtk_label_new (_("Scale:"));
	gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 4);
    scale = gtk_combo_box_new_text ();
	for (i = 0; view_scale_list[i] > 0.0; i++) {
		if (view_scale_list[i] < 0.1) sprintf (txt, _("%.4g m/px"), 1000.0*view_scale_list[i]);
		else sprintf (txt, _("%.4g km/px"), view_scale_list[i]);
		gtk_combo_box_append_text (GTK_COMBO_BOX(scale), txt);
	}
	gtk_combo_box_set_active (GTK_COMBO_BOX(scale), view_scale);
	g_signal_connect (G_OBJECT (scale), "changed", G_CALLBACK (main_wnd_scale_change), NULL);
	gtk_box_pack_start (GTK_BOX (box), scale, FALSE, FALSE, 4);

	widget = gtk_vseparator_new();
	gtk_box_pack_start (GTK_BOX (box), widget, FALSE, FALSE, 4);

	/* progress bar */
	progressbar = gtk_progress_bar_new ();
 	gtk_progress_bar_set_ellipsize (GTK_PROGRESS_BAR(progressbar), PANGO_ELLIPSIZE_END);
	gtk_box_pack_start (GTK_BOX (box), progressbar, TRUE, TRUE, 0);
	
	/* statusbar */	
	statusbar = gtk_statusbar_new ();
	status_bar_contextid = gtk_statusbar_get_context_id (GTK_STATUSBAR (statusbar), "");
	gtk_box_pack_start (GTK_BOX (box), statusbar, TRUE, TRUE, 0);
	g_timeout_add (10, main_wnd_cyclicloop, 0);
	
	gtk_box_pack_start (GTK_BOX (main_vbox), box, FALSE, TRUE, 0);

	return window;
};


void main_wnd_startall () {
	g_timeout_add (10, main_wnd_cyclicloop, 0);
};


/*
 * some other usefull functions..
 *
 *
 */
void widget_change_fontsize (GtkWidget *w, float s) {
	PangoFontDescription *pfd;
	pfd = pango_context_get_font_description(gtk_widget_get_pango_context(w));
	pango_font_description_set_size(pfd, ((int)((float)pango_font_description_get_size (pfd)*(float)s)));
	gtk_widget_modify_font(w, pfd);
};


void main_wnd_loop (int cur, int max) {
	static time_t to;
	time_t t1;

	t1 = time (NULL);
	if (to != t1) {
		main_wnd_status (NULL, 0);
		to = t1;
	}

	if (!gtk_running) return;
	while (gtk_events_pending())
		gtk_main_iteration();	
};


void main_wnd_update () {
	main_wnd_status(NULL, 0);

	if (!gtk_running) return;
	while (gtk_events_pending())
		gtk_main_iteration();	
};
