/***************************************************************************
 *            gtk_map_screen.c
 *
 *  Thu Jul 19 19:00:10 2007
 *  Copyright  2007  Steffen Pohle
 *  steffen@gulpe.de
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "map.h"
#include "gtk_port.h"


/*
 * map drawarea
 */
GtkWidget *map_da = NULL;
GdkGC  *gc;
GdkGCValues gcvalues;
GdkColormap *colormap = NULL;
time_t buttonpresstime;
int buttonpressed = 0;
iPoint buttonposition = {0 , 0};


/*
 * draw event callback
 */
gboolean map_da_expose_event (GtkWidget *widget, GdkEventExpose *event, gpointer data) {
	draw ();
	return FALSE;
};


/*
 * check if a button was pressed and repaint the cursor depending on the
 * time how long we pressed the button.       call from cyclic loop
 */
void map_da_button_stillpressed () {
	time_t t;
	static int preventloop = 0;

	if (!buttonpressed) {
		preventloop = 0;
		return;
	}
	if (preventloop) return;

	t = time (NULL);
	if (t - buttonpresstime > 1) {
		int x = buttonposition.x - (map_da->allocation.width/2);
		int y = buttonposition.y - (map_da->allocation.height/2);
		float lon, lat;

		lat = -map_km2lat(((float)y) * view_scale_list[view_scale]);
		lon = map_km2lon(((float)x) * view_scale_list[view_scale], view_lat + lat);
		
		draw_set_pointerpos (view_lon + lon, view_lat + lat, 1);
		draw ();
		preventloop = 1;
	}
};


gboolean map_da_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	buttonpresstime = time (NULL);
	buttonposition.x = event->x;
	buttonposition.y = event->y;
	buttonpressed = 1;

	return FALSE;
};


gboolean map_da_button_release_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	time_t btnreltime = time (NULL);

	int x = event->x - (map_da->allocation.width/2);
	int y = event->y - (map_da->allocation.height/2);
	float lon, lat;

	if (!buttonpressed) return FALSE;	/* button is no longer pressed */
	buttonpressed = 0;

	if (draw_button_check (event->x, event->y) != DRAW_BTN_NONE) return FALSE;
	
	lat = -map_km2lat(((float)y) * view_scale_list[view_scale]);
	lon = map_km2lon(((float)x) * view_scale_list[view_scale], view_lat + lat);

	/* short time button pressing */
	if (btnreltime - buttonpresstime <= 1) {
		if (event->button == 1) {
			view_lat = view_lat + lat;
			view_lon = view_lon + lon;
		}
		else if (event->button == 3) {
			selected_way_pos[0].lon = view_lon + lon;
			selected_way_pos[0].lat = view_lat + lat;
			selected_way = map_way_findclosest (&selected_way_pos[0], &selected_way_pos[1], 0, selected_way_name, MAP_W_NAMELEN);
		}
	}

	/* long time button pressed */
	else {
		d_printf ("button pressed long time.");
	}
	
	main_wnd_update ();
	draw ();

	return FALSE;
};


gboolean map_da_motion_notify_event (GtkWidget *widget, GdkEventMotion *event, gpointer user_data) {
	int x = event->x - (map_da->allocation.width/2);
	int y = event->y - (map_da->allocation.height/2);
	float lon, lat;
	char sbtext[255];
	unsigned long long int oldselwayid = selected_way;

	lat = -map_km2lat(((float)y) * view_scale_list[view_scale]);
	lon = map_km2lon(((float)x) * view_scale_list[view_scale], view_lat + lat);

	draw_set_pointerpos (view_lon + lon, view_lat + lat, 0);
	snprintf (sbtext, 255, _("%5.5f"), view_lon + lon);
	gtk_entry_set_text (GTK_ENTRY(positionlon), sbtext);
	snprintf (sbtext, 255, _("%5.5f"), view_lat + lat);
	gtk_entry_set_text (GTK_ENTRY(positionlat), sbtext);
 
	if (!gdk_events_pending ()) {
		if (selected_way != oldselwayid) draw_redrawmap ();
		draw ();
	}

	return FALSE;
};


gboolean map_da_key_release_event (GtkWidget *widget, GdkEventKey *event, gpointer user_data) {
	if (event->keyval == '+') draw_setscale (view_scale - 1);
	if (event->keyval == '-') draw_setscale (view_scale + 1);

	main_wnd_update ();
	draw ();

	return FALSE;
};


gboolean map_da_scroll_event (GtkWidget *widget, GdkEventScroll *event, gpointer user_data) {
	int x = event->x - (map_da->allocation.width/2);
	int y = event->y - (map_da->allocation.height/2);
	float lon, lat;

	lat = -map_km2lat(((float)y) * view_scale_list[view_scale]);
	lon = map_km2lon(((float)x) * view_scale_list[view_scale], lat);

	view_lat = view_lat + lat;
	view_lon = view_lon + lon;

	if (event->direction == GDK_SCROLL_UP) draw_setscale (view_scale - 1);
	if (event->direction == GDK_SCROLL_DOWN) draw_setscale (view_scale + 1);

	main_wnd_update ();
	draw ();
	
	return FALSE;
};


/*
 * create the map drawarea
 */
GtkWidget *map_da_create () {
	map_da = gtk_drawing_area_new ();
	gtk_widget_set_size_request (map_da, 100, 100);
	gtk_widget_set_events (map_da, GDK_EXPOSURE_MASK | GDK_LEAVE_NOTIFY_MASK
			| GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_KEY_PRESS_MASK
			| GDK_KEY_RELEASE_MASK	| GDK_POINTER_MOTION_MASK );
	GTK_WIDGET_SET_FLAGS(map_da, GTK_CAN_FOCUS);	
	g_signal_connect (G_OBJECT (map_da), "expose_event", G_CALLBACK (map_da_expose_event), NULL);
	g_signal_connect (G_OBJECT (map_da), "button_press_event", G_CALLBACK (map_da_button_press_event), NULL);
	g_signal_connect (G_OBJECT (map_da), "button_release_event", G_CALLBACK (map_da_button_release_event), NULL);
	g_signal_connect (G_OBJECT (map_da), "scroll_event", G_CALLBACK (map_da_scroll_event), NULL);
	g_signal_connect (G_OBJECT (map_da), "key_release_event", G_CALLBACK (map_da_key_release_event), NULL);
	g_signal_connect (G_OBJECT (map_da), "motion_notify_event", G_CALLBACK (map_da_motion_notify_event), NULL);

	buttonpresstime = time (NULL);
	buttonpressed = 0;

	return (void *) map_da;
};


