/***************************************************************************
 *            map.c
 *
 *  Thu Jul 19 19:00:10 2007
 *  Copyright  2007  Steffen Pohle
 *  steffen@gulpe.de
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <math.h>
#include "map.h"
#include "memoryleak.h"


struct map_hash **mhash = NULL;
int mhash_max = 0;
struct map_lsstat *mlsstat = NULL;
int map_busy = 0;
extern int map_search_fd;



uint64_t selected_way = 0;
struct map_pos selected_way_pos[2];
char selected_way_name[MAP_W_NAMELEN] = "\0";



/******************************************************
 * Initialize the map data, allocate memory and
 * increase the memory.
 */
void map_init () {
	int i, j;

	if (mlsstat == NULL)
		mlsstat = (struct map_lsstat*) ml_malloc (MAP_LSSTAT_MAX*sizeof (struct map_lsstat));
	for (i = 0; i < MAP_LSSTAT_MAX; i++) {
		mlsstat[i].fname[0] = '\0';
		mlsstat[i].fnametmp[0] = '\0';
		for (j = 0; j < MAP_LSHASHFACTOR*MAP_LSHASHFACTOR; j++) {
			mlsstat[i].r_lshpos[j].pos = 0;
			mlsstat[i].r_lshpos[j].size = 0;
			mlsstat[i].w_lshpos[j].pos = 0;
			mlsstat[i].w_lshpos[j].size = 0;
			if (j < MAP_LSFREECNT) {
				mlsstat[i].w_free[j].pos = 0;
				mlsstat[i].w_free[j].size = 0;
			}
		}
		mlsstat[i].w_fd = 0;
		mlsstat[i].r_fd = 0;
	}

	mhash_max = MAP_HASH_MAX;
	mhash = (struct map_hash**) ml_malloc (sizeof (void*) * (mhash_max));

	for (i = 0; i < mhash_max; i++) 
		mhash[i] = NULL;
};


void map_clear () {
	int i;

	d_printf ("map_clean enter");
	d_printf ("map_clean mhash reset");
	if (mhash != NULL) {
		for (i = 0; i < mhash_max; i++)
			if (mhash[i]) {
				ml_free (mhash[i]);
				mhash[i] = NULL;
			}
		ml_free (mhash);
		mhash = NULL;
		mhash_max = 0;
	}
	d_printf ("map_clean mlsstat reset");
	if (mlsstat) {
		ml_free (mlsstat);
		mlsstat = NULL;
	}
	if (map_search_fd) {
		close (map_search_fd);
		map_search_fd = 0;
	}

	d_printf ("map init");
	map_init ();
	d_printf ("map_clean left");
};


/*********************************************************************************
 *
 * convert input text to geo-data needed for the programm
 * RETURN 1 on success, 0 on error
 */
int map_text2geo (char *text, float *dest) {
	int i;
	int type = 0;	// 0 = grad, 1=min, 2=sec
	char ctab[3][32];
	int itab[3] = { 0, 0, 0};
	float ftab[3] = {0.0, 0.0, 0.0 };

	memset (ctab, 0x0, sizeof (ctab));
	if (text == NULL || dest == NULL) return 0;
	for (i = 0; itab[type] < 32 && i < 63; i++) {
		if (text[i] == '\0') break;
		else if (text[i] == '"') type = 1;
		else if (text[i] == '\'') type = 2;
		else 
			ctab[type][itab[type]++] = text[i];
	}

	for (i = 0; i < 3; i++)
		ftab[i] = atof(ctab[i]);

	*dest = ftab[0] + ftab[1]/60.0 + ftab[2]/3600.0;
	return 1;
};


/*********************************************************************************
 *
 * convert between stretched km/map and geographial coordinates
 *
 */
float map_lon2km (float lon, float lat) {
	return (lon * cosf(M_PI * lat/180.0) * M_PI * MAP_EARTHRADIUS/180.0);
};

float map_lat2km (float lat) {
	return (lat * M_PI * MAP_EARTHRADIUS/180.0);
};

float map_km2lon (float km, float lat) {
	return (180.0 * km / (cosf(M_PI * lat/180.0) * M_PI * MAP_EARTHRADIUS));
};

float map_km2lat (float km) {
	return (180.0 * km / (M_PI * MAP_EARTHRADIUS));
};



/*********************************************************************************
 *
 * convert between integer lon and lat and the float lon lat values
 * needed for handling with the hash tables
 *
 */
int map_geo2igeo (float f) {
	if (f < -720.0 || f > 720.0) {
		d_printf ("map_geo2igeo f out of range. f:%f", f);
		d_print_backtrace ();
		f = 0.0;
	}
	
	f = f + 180.0;

	while (f < 0.0) f = f + 360.0;
	while (f >= 360.0) f = f - 360.0;

	return ((int)(f*MAP_HASH_DEGFACTOR));
};


float map_igeo2geo (int i) {
	return ((((float)i)/MAP_HASH_DEGFACTOR)-180.0);
};




/******************************************************************************
 *
 * Debugging Functions..
 *
 */
void map_way_print (struct map_way *mw) {
/*	int i;
	unsigned long long int id;

	printf ("DEBUG: Way ID:\"%-12llu\".%d  Pos[%.4f,%.4f] Flags:%d , next[%.4f,%.4f], prev[%.4f,%.4f]\n   ", mw->id,mw->subid, mw->p.lon, mw->p.lat, mw->flags, mw->np.lon, mw->np.lat, mw->pp.lon, mw->pp.lat);
	for (i = 0; i < MAP_N_REFS; i++) if (mw->n_id[i] != 0) {
		id = mw->n_id[i];
		printf ("%d:'%-12llu' ", i, id);
	}
	printf ("\n");
*/
};



/*
 * Display Map Informations
 */
void map_debug () {
	int i;
	d_printf ("**** Mapinformation");
	d_printf ("Hash Information:");
	for (i = 0; i < mhash_max; i++) if (mhash[i]) {
		d_printf ("  mhash[%d] : prt:%p", i, mhash[i]);
	}
};
