/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * ncurses_main.c
 * Copyright (C) Steffen Pohle 2009 <steffen@gulpe.de>
 * 
 * main.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * main.c is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/*
 * convert_main.c
 * Copyright (C) Steffen Pohle 2009 <steffen@gulpe.de>
 * 
 * main.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * main.c is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include <math.h>
#include <limits.h>
#include <curses.h>

#include "memoryleak.h"
#include "map.h"
#include "convert_port.h"

char curfile[LEN_FILENAME];

/* call this function always at the end of programm */
void main_wnd_quit() {
	endwin();
}


int main (int argc, char *argv[]) {
	struct stat sb;
	int i = 0, c;

	ml_init();
#ifdef ENABLE_NLS
	bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
	textdomain (GETTEXT_PACKAGE);
#endif

	/* check for the cache path and the map path */
	if (stat(cachepath, &sb) == -1) {
		mkdir (cachepath, 0755);
	}

	if (stat(datapath, &sb) == -1) {
		mkdir (datapath, 0755);
	}
	
	config_init ();
	config_load ();

	map_init ();
	main_wnd_startall ();
	
	if (argc > 1) {
		strncpy (curfile, argv[1], LEN_FILENAME);
	}
	else {		
		strncpy (curfile, "/home/steffen/Daten/Programmierung/gOSMroute/sample.osm", LEN_FILENAME);
	}
	
	osm_loadfile (curfile);
	map_save_all ();
	color_set(2, 0);	
	mvaddstr(1, 18, "press any key to quit programm"); 
	refresh();
	do {
		c = getch ();
		refresh();
		usleep (1000);
	} while ( c == -1 );

	return 0;
};


void main_wnd_startall () {
	initscr();
	atexit(main_wnd_quit);
	noecho ();
	curs_set(0);
	nodelay (stdscr, 1);
	start_color();
	init_pair(1, COLOR_GREEN, COLOR_BLACK); 
	init_pair(2, COLOR_WHITE, COLOR_BLACK); 
	init_pair(3, COLOR_CYAN, COLOR_BLACK); 
	bkgd(COLOR_PAIR(1));
};


void main_wnd_loop (long long int cur, long long int max) {
	struct memoryleak_info mi;
	static time_t timeout = 0;
	int ways, hash, i, x, y;
	
	if (timeout+1 > time (NULL)) return;
	timeout = time(NULL);
	
	ml_get_stats (&mi);

	clear();
	attrset(COLOR_PAIR(1));
	mvprintw(0,1, "Converting %s file.", curfile);

	attrset(A_BOLD | COLOR_PAIR(2));
	mvprintw(0,12, "%s", curfile);
	
	attrset(COLOR_PAIR(3));
	mvprintw(1,1, "---------------------------------------------------------------------");
	mvprintw(2,1, " Current File Pos:         %%            |Memory Used:");
	mvprintw(3,1, "            nodes:                      |     Blocks:");
	mvprintw(4,1, "             ways:                      |");
	
	attrset(A_BOLD | COLOR_PAIR(2));
	mvprintw(2, 19, "%3.3f", osm_file_pos);
	mvprintw(3, 19, "%lld", osm_nodes_cnt);
	mvprintw(4, 19, "%lld", osm_ways_cnt);	

	mvprintw(5, 1, "Status: %-5d of %-5d", cur, max);	

	if (mi.size < 10*1024)	mvprintw(2, 55, "%lld bytes", mi.size);
	else if (mi.size < 10*1024*1024) mvprintw(2, 55, "%lld kbytes", mi.size/1024);
	else mvprintw(2, 55, "%lld Mbytes", mi.size/(1024*1024));
	mvprintw(3, 55, "%lld", mi.blocks);
	
	if (max == -2) {
		/* searching way ... give informations */
		attrset(A_BOLD | COLOR_PAIR(1));
		mvprintw(5, 40, "adding nodes to the way id:");
		attrset(A_BOLD | COLOR_PAIR(2));
		mvprintw(5, 68, "%lld", cur);
	}

	osm_convert_info ();

	attrset(A_BOLD | COLOR_PAIR(3));
	mvprintw(6,50, "MAP Hash Information");
	attrset(COLOR_PAIR(3));
	mvprintw(7, 41, "+------------------------------------+");
	for (i = 0; i < 12; i++) mvprintw(8+i, 41, "|                                    |");
	mvprintw(19, 41, "+------------------------------------+");
	mvprintw(20, 41, " hash cnt:       ways cnt:");
	attrset(A_BOLD | COLOR_PAIR(2));
	for (i = 0, hash = 0, ways = 0; i < mhash_max; i++) {
		if (mhash[i]) {
			x = (float) (36*(map_igeo2geo(mhash[i]->pos.ilon)+160.0)/360.0);
			y = (float) (10*((-map_igeo2geo(mhash[i]->pos.ilat))+90.0)/180.0);
			mvprintw(8+y, 42+x, "*");

			hash++;
			ways += mhash[i]->wayscnt;
		}
	}
	mvprintw(20, 51, "%d", hash);
	mvprintw(20, 67, "%d", ways);
	
	refresh ();
};


void main_wnd_status (char *text, int progress) {
};


void main_wnd_update () {
};

void draw () {
};
