/***************************************************************************
 *            system.h
 *
 *  Copyright  2009 - Steffen Pohle
 *  steffen@gulpe.de
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _SYSTEM_H_
#define _SYSTEM_H_

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#if !defined(__MINGW32CE__) && !defined(_WIN32_WCE) && !defined(__MINGW32__)
  #define POINTERALIGNMENT 4
#include <sys/wait.h>
  #include <errno.h>
  #include <execinfo.h>
#else
  #if defined(__MINGW32CE__) || defined(_WIN32_WCE) 
    #define POINTERALIGNMENT 8
    #define errno 0
    #define strerror(_t) "no strerror support in winCE"
  #else
    #define POINTERALIGNMENT 4
  #endif
#endif
#include <math.h>
#include <limits.h>
#include <float.h>
#include <stdarg.h>


#define SETFLAG(__flag, __opt) (__flag |= __opt)
#define DELFLAG(__flag, __opt) (__flag &= (0x0FFFF - __opt))

struct s_execcall {
	int used;
	pid_t pid;
};

extern int cmpf (float f1, float f2);
extern int cmpd (double d1, double d2);

extern void d_printf (char *fmt,...);
extern void d_print_init();

extern void errorexit (int nr);
extern void d_print_backtrace ();

extern int execcall (struct s_execcall *p, char *prg, char **argv);
extern int execwait (struct s_execcall *p);

extern unsigned long long int getticks ();
extern char **get_backtrace (int *cnt);

extern char *mempush (void *mem, void *source, int size);
extern char *mempop (void *mem, void *dest, int size);
extern char *memstrpop (void *mem, void *dest, int *size);

struct stat *file_exist (char *fn);
#endif
